from jinja2 import Template


page_template = Template("""{{ '{% load static from staticfiles %}' }}
{{ '{% load cache %}' }}

{{ '{% comment %}' }}
    Template for {{ name }}
    Describe the template here.
{{ '{% endcomment %}' }}

{{ '{% block header %}{% endblock header %}' }}

{{ '{% block body %}{% endblock body %}' }}

{{ '{% block footer %}{% endblock footer %}' }}

{{ '{% block scripts %}{% endblock scripts %}' }}
""")
